/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Clippings.
 *
 * The Initial Developer of the Original Code is 
 * Alex Eng <ateng@users.sourceforge.net>.
 * Portions created by the Initial Developer are Copyright (C) 2005-2008
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * ***** END LICENSE BLOCK ***** */

//
// Integration with host application
//


var gClippings = {
  dataSrcInitialized:    false,
  showDialog:            true,
  showPasteOpts:         false,
  appUtils:              null,
  clippingsSvc:          null,
  strBundle:             null,
  _clippingsListener:    null,
  _clippingsToolbar:     null,
  _isErrMenuItemVisible: false,
  _clippingSubst:        null,
  _ds:                   null,
  _initContextMenuItem:  null,

  statusBarObserver: {
    onDragOver: function (aEvent, aFlavour, aSession)
    {
      aEvent.target.setAttribute("ae-dnd", "true");
    },

    onDragExit: function (aEvent, aSession)
    {
      aEvent.target.removeAttribute("ae-dnd");
    },

    onDrop: function (aEvent, aTransferData, aSession)
    {
      if (! gClippings.dataSrcInitialized) {
	// The RDF data source has to be initialized if it has not already been
	// done so, otherwise RDF node creation will fail, and the new
	// Clippings entry will never be created.
	// This initialization is done in the code for the Clippings popup
	// menu's `onpopupshowing' event handler.
	gClippings.initClippingsPopup(document.getElementById("ae-clippings-popup-1"), document.getElementById("ae-clippings-menu-1"));
      }

      var text = aTransferData.data;
      var result = gClippings.clippingsSvc.newClippingFromText(text, gClippings.showDialog, window, null, false);
      if (result) {
	window.setTimeout("gClippings.saveClippings();", 100);
      }
    },

    getSupportedFlavours: function ()
    {
      var flavours = new FlavourSet();
      flavours.appendFlavour("text/unicode");
      return flavours;
    }
  },


  // Method handleEvent() effectively makes gClippings an implementation of
  // the EventListener interface; therefore it can be passed as the listener
  // argument to window.addEventListener() and window.removeEventListener()

  handleEvent: function (aEvent)
  {
    // When this method is invoked, 'this' will not refer to gClippings
    var _this = gClippings;

    if (aEvent.type == "load") {
      _this.initClippings();
    }
    else if (aEvent.type == "unload") {
      _this.unload();
      window.removeEventListener("load", _this, false);
      window.removeEventListener("unload", _this, false);
      window.removeEventListener("focus", _this, false);

      var hostAppCxtMenu = document.getElementById("msgComposeContext");
      hostAppCxtMenu.removeEventListener("popupshowing", 
					 _this._initContextMenuItem,
					 false);
    }
    else if (aEvent.type == "focus") {
      _this.focusHostAppWnd();
    }
  },


  //
  // Methods invoked by overlay code
  //

  alert: function (aMessage)
  {
    var title = this.strBundle.getString('appName');
    this.appUtils.alertEx(title, aMessage);
  },


  newFromClipboard: function () 
  {
    if (! this.dataSrcInitialized) {
      // The RDF data source has to be initialized if it has not already been
      // done so, otherwise RDF node creation will fail, and the new Clippings
      // entry will never be created.
      // This initialization is done in the code for the Clippings popup menu's
      // `onpopupshowing' event handler.
      this.initClippingsPopup(document.getElementById("ae-clippings-popup-1"),
			      document.getElementById("ae-clippings-menu-1"));
    }

    var txt = this.appUtils.getTextFromClipboard();
    if (! txt) {
      this.alert(this.strBundle.getString('errorNoClipboardContent'));
      return;
    }

    var result = this.clippingsSvc.newClippingFromText(txt, this.showDialog, window, null, false);
    if (result) {
      window.setTimeout("gClippings.saveClippings();", 1);
    }
  },


  getSelectedText: function ()
  {
    var rv;
    var focusedWnd = document.commandDispatcher.focusedWindow;
    rv = focusedWnd.getSelection().toString();
    return rv;
  },


  extNewClipping: function ()
  {
    // This function is meant to be invoked by the Clippings toolbar
    var cmdCopy = document.getElementById("cmd_copy");
    if (cmdCopy.getAttribute("disabled")) {
      this.appUtils.beep();
      return;
    }

    goDoCommand("cmd_copy");
    var selection = this.appUtils.getTextFromClipboard();

    if (selection) {
      var result = this.clippingsSvc.newClippingFromText(selection, this.showDialog, window, null, false);
      if (result) {
	window.setTimeout("gClippings.saveClippings();", 1);
      }
    }
  },


  newFromSelection: function ()
  {
    if (! this.dataSrcInitialized) {
      this.initClippingsPopup(document.getElementById("ae-clippings-popup-1"),
			      document.getElementById("ae-clippings-menu-1"));
    }

    // Must explicitly close the message compose context menu - otherwise it
    // will reappear while the New Clipping dialog is open if the Clippings 
    // submenu needs to be rebuilt.
    var cxtMenu = document.getElementById("msgComposeContext");
    cxtMenu.hidePopup();

    var selection = this.getSelectedText();
    if (selection) {
      var result = this.clippingsSvc.newClippingFromText(selection, this.showDialog, window, null, false);
      if (result) {
	window.setTimeout("gClippings.saveClippings();", 1);
      }
    }
    else {
      var strBundle = document.getElementById("ae-clippings-strings");
      this.alert(this.strBundle.getString('errorNoSelection'));
    }
  },


  openClippingsManager: function () 
  {
    var wnd = window.open("chrome://clippings/content/clippings.xul",
			  "ae_clippings_wndobj", "chrome,resizable");
    wnd.focus();
  },


  initToolbarCmd: function (aID)
  {
    var isToolbarOpen = this._clippingsToolbar.isOpen();
    var toolbarCmd = document.getElementById(aID);
    toolbarCmd.setAttribute("checked", isToolbarOpen);
  },


  toggleClippingsToolbar: function ()
  {
    if (this._clippingsToolbar.isOpen()) {
      this._clippingsToolbar.close();
    }
    else {
      var toolbarWnd = this._clippingsToolbar.open();
      toolbarWnd.focus();
    }
  },


  toggleShowPasteOptions: function ()
  {
    this.showPasteOpts = !this.showPasteOpts;
    document.getElementById("ae_clippings_show_paste_options").setAttribute("checked", this.showPasteOpts);
  },


  initClippingsPopup: function (aPopup, aMenu) 
  {
    var err = false;
    var dsURL = this.appUtils.getDataSourcePathURL() + this.appUtils.CLIPDAT_FILE_NAME;
    try {
      this._ds = this.clippingsSvc.getDataSource(dsURL);
    }
    catch (e if e.result == Components.results.NS_ERROR_OUT_OF_MEMORY) {
      err = this.strBundle.getString("errorOutOfMemory");
    }
    catch (e if e.result == Components.results.NS_ERROR_FILE_ACCESS_DENIED) {
      err = String.ae_fmt("%s: %s",
			  this.strBundle.getString("errorAccessDenied"),
			  this.appUtils.CLIPDAT_FILE_NAME);
    }
    catch (e if e.result == Components.results.NS_ERROR_FILE_IS_LOCKED) {
      err = String.ae_fmt("%s: %s",
			  this.strBundle.getString("errorFileLocked"),
			  this.appUtils.CLIPDAT_FILE_NAME);
    }
    catch (e if e.result == Components.results.NS_ERROR_FILE_TOO_BIG) {
      err = String.ae_fmt("%s: %s",
			  this.strBundle.getString("errorFileTooBig"),
			  this.appUtils.CLIPDAT_FILE_NAME);
    }
    catch (e) {
      // File is corrupt - open Clippings Manager and perform recovery.
      err = 888;
    }

    var errorMenuItem = document.getElementById("ae-clippings-error-menuitem");

    if (err) {
      // Append a new menu item that lets user see more details of error.
      if (errorMenuItem) {
	return;
      }

      var errorCmd = document.createElement("menuitem");
      errorCmd.id = "ae-clippings-error-menuitem";
      errorCmd.setAttribute("label", this.strBundle.getString("errorMenu"));
      errorCmd.style.fontWeight = "bold";

      var func;
      if (err == 888) {
	func = function () { gClippings.openClippingsManager() };
      }
      else {
	func = function () {
	  gClippings.appUtils.alertEx(gClippings.strBundle.getString('appName'), err);
	};
      }
      errorCmd.addEventListener("command", func, false);
      aPopup.appendChild(errorCmd);
      this._isErrMenuItemVisible = true;
      return;
    }
    else {
      // Remove error menu item if error condition no longer exists
      if (errorMenuItem) {
	aPopup.removeChild(errorMenuItem);
	this._isErrMenuItemVisible = false;
      }
    }

    aMenu.database.AddDataSource(this._ds);
    aMenu.builder.rebuild();

    this.dataSrcInitialized = true;
    this.appUtils.log("gClippings.initClippingsPopup(): Data source initialization completed.");
  },


  insertClippingText: function (aURI, aName, aText) 
  {
    // Must explicitly close the message compose context menu - otherwise it
    // may reappear while the paste options dialog is open.
    var cxtMenu = document.getElementById("msgComposeContext");
    cxtMenu.hidePopup();

    var parentFolderURI = this.clippingsSvc.getParent(aURI);
    var folderName = this.clippingsSvc.getName(parentFolderURI);
    var clippingInfo = new aeClippingInfo(aURI, aName, aText, folderName);
    var txt = this._clippingSubst.processClippingText(clippingInfo);
    this.appUtils.copyTextToClipboard(txt);

    if (this.showPasteOpts) {
      var dlgArgs = { userCancel: null };
      window.openDialog("chrome://clippings/content/pasteOptions.xul", "ae_clippings_pasteopt_dlg", "chrome,centerscreen,modal", dlgArgs);

      if (dlgArgs.userCancel) {
	return;
      }

      if (dlgArgs.pasteOption == 1) {
	goDoCommand("cmd_pasteQuote");

	// gMsgCompose defined in <chrome://messenger/content/messengercompose/
	// MsgComposeCommands.js>.  It is an instance of nsIMsgCompose,
	// progid "@mozilla.org/messengercompose/compose;1"
	// Line rewrapping not necessary if composing in HTML format
	if (! gMsgCompose.composeHTML) {
	  goDoCommand("cmd_rewrap");
	}
	return;
      }
    }

    window.setTimeout("gClippings._pasteClipping()", 8);
  },


  _pasteClipping: function ()
  {
    try {
      // Paste clipping.  The following function is defined in
      // "chrome://global/content/globalOverlay.js"
      goDoCommand('cmd_paste');
      // SIDE EFFECT: The clipping text will remain on the system clipboard.
    }
    catch (e) {
      // Exception thrown if command is disabled or not applicable
      this.appUtils.beep();
    }
  },


  keyboardInsertClipping: function ()
  {
    if (this.appUtils.isCommonDataSourceEnabled()) {
      var clippingsMenu1 = document.getElementById("ae-clippings-menu-1");
      clippingsMenu1.builder.refresh();
      clippingsMenu1.builder.rebuild();
    } 

    const KEY_ESC = 27;
    var dlgArgs = { key: null };
    var dlg = window.openDialog("chrome://clippings/content/clippingKey.xul",
				"clipkey_dlg", "modal,centerscreen", dlgArgs);

    if (dlgArgs.key && dlgArgs.key != KEY_ESC) {
      var key = String.fromCharCode(dlgArgs.key);
      var keyDict = this.clippingsSvc.getShortcutKeyDict();

      if (! keyDict.hasKey(key)) {
	this.appUtils.beep();
	return;
      }

      try {
	var valueStr = keyDict.getValue(key);
      }
      catch (e) {}

      valueStr = valueStr.QueryInterface(Components.interfaces.nsISupportsString);
      var clippingURI = valueStr.data;
      this.insertClippingText(clippingURI,
			      this.clippingsSvc.getName(clippingURI),
			      this.clippingsSvc.getText(clippingURI));
    }
  },


  saveClippings: function () 
  {
    var title = this.strBundle.getString('appName');
    try {
      this.clippingsSvc.flushDataSrc();
    }
    catch (e if e.result == Components.results.NS_ERROR_NOT_INITIALIZED) {
      this.appUtils.alertEx(title, this.strBundle.getString("errorSaveFailedDSNotInitialized"));
      return;
	   }
    catch (e if e.result == Components.results.NS_ERROR_OUT_OF_MEMORY) {
      this.appUtils.alertEx(title, this.strBundle.getString("errorOutOfMemory"));
      return;
    }
    catch (e if e.result == Components.results.NS_ERROR_FILE_ACCESS_DENIED) {
      var msg = String.ae_fmt("%s: %s",
			      this.strBundle.getString("errorAccessDenied"),
			      this.appUtils.CLIPDAT_FILE_NAME);
      this.appUtils.alertEx(title, msg);
      return;
    }
    catch (e if e.result == Components.results.NS_ERROR_FILE_IS_LOCKED) {
      var msg = String.ae_fmt("%s: %s",
			      this.strBundle.getString("errorFileLocked"),
			      this.appUtils.CLIPDAT_FILE_NAME);
      this.appUtils.alertEx(title, msg);
      return;
    }
    catch (e if e.result == Components.results.NS_ERROR_FILE_TOO_BIG) {
      var msg = String.ae_fmt("%s: %s",
			      this.strBundle.getString("errorFileTooBig"),
			      this.appUtils.CLIPDAT_FILE_NAME);
      this.appUtils.alertEx(title, msg);
      return;
    }
    catch (e if e.result == Components.results.NS_ERROR_FILE_READ_ONLY) {
      var msg = String.ae_fmt("%s: %s",
			      this.strBundle.getString('errorFileReadOnly'),
			      this.appUtils.CLIPDAT_FILE_NAME);
      this.appUtils.alertEx(title, msg);
      return;
    }
    catch (e if e.result == Components.results.NS_ERROR_FILE_DISK_FULL) {
      var msg = String.ae_fmt("%s: %s",
			      this.strBundle.getString('errorDiskFull'),
			      this.appUtils.CLIPDAT_FILE_NAME);
      this.appUtils.alertEx(title, msg);
      return;
    }
    catch (e) {
      this.appUtils.alertEx(title, this.strBundle.getString("alertSaveFailed"));
      return;
    }
  },


  //
  // Browser window and Clippings menu initialization
  //

  isClippingsInitialized: false,

  initClippings: function ()
  {   
    // Workaround to this init function being called multiple times
    if (this.isClippingsInitialized) {
      return;
    }

    this.appUtils = new aeClippingsUtils();
    this.strBundle = document.getElementById("ae-clippings-strings");
    this._clippingSubst = new aeClippingSubst(this.strBundle);

    try {
      this.clippingsSvc = Components.classes["clippings@mozdev.org/clippings;1"].getService(Components.interfaces.nsIClippingsService);
    }
    catch (e) {
      this.alert(e);
    }

    // String used for empty (null) clippings
    this.clippingsSvc.setEmptyClippingString(this.strBundle.getString('emptyClippingLabel'));

    if (this.appUtils.getPref("clippings.first_run") == true) {
      this.appUtils.setPref("clippings.first_run", false);
    }

    // First-run initialization after upgrade from 2.x -> 3.0
    if (this.appUtils.getPref("clippings.v3.first_run") == true) {
      var clippings3 = new aeClippings3(this.clippingsSvc, this.strBundle);
      var initFinished = clippings3.startInit();
      if (initFinished) {
	this.appUtils.setPref("clippings.v3.first_run", false);
      }
    }

    var commonDSVerifyMsg = "";
    if (this.appUtils.getPref("clippings.datasource.common")) {
      commonDSVerifyMsg = "Enabled common data source between Firefox and Thunderbird.";
    }

    // Clippings backup
    var backupDirURL = this.appUtils.getDataSourcePathURL() + this.appUtils.BACKUP_DIR_NAME;
    this.clippingsSvc.setBackupDir(backupDirURL);
    this.clippingsSvc.setMaxBackupFiles(this.appUtils.getPref('clippings.backup.maxfiles'));

    // Initializing data source on Clippings context menu
    var menu = document.getElementById("ae-clippings-menu-1");
    var popup = document.getElementById("ae-clippings-popup-1");
    this.initClippingsPopup(popup, menu);

    this.appUtils.log(String.ae_fmt("gClippings.initClippings(): Initializing Clippings integration with host app window: %s\n%s", window.location.href, commonDSVerifyMsg));

    // Add null clipping to root folder if there are no items
    if (this.appUtils.getPref("clippings.datasource.process_root") == true) {
      this.clippingsSvc.processRootFolder();
      this.appUtils.setPref("clippings.datasource.process_root", false);
    }

    // Attaching event handler to context menu 
    var hostAppContextMenu = document.getElementById("msgComposeContext");
    this._initContextMenuItem = {
      handleEvent: function (aEvent) {
	if (aEvent.target.id != "msgComposeContext") return;
	gClippings.initContextMenuItem();
      }
    };

    hostAppContextMenu.addEventListener("popupshowing", 
					this._initContextMenuItem, false);

    this._clippingsListener = {
      origin:  this.clippingsSvc.ORIGIN_HOSTAPP,

      migrationDone: function (aDataSrcURL) {
	var menu = document.getElementById("ae-clippings-menu-1");
	var popup = document.getElementById("ae-clippings-popup-1");

	// Reinitialize Clippings menu so that it points to the correct datasrc
	menu.database.RemoveDataSource(gClippings._ds);
	gClippings.initClippingsPopup(popup, menu);
      },

      newFolderCreated:    function (aFolderURI) {},
      newClippingCreated:  function (aClippingURI) {},
      importDone:          function (aNumItems) {}
    };

    this.clippingsSvc.addListener(this._clippingsListener);

    // Set behaviour of "New Clipping" commands - prompt vs. silent operation
    this.showDialog = true;
    try {
      this.showDialog = !(this.appUtils.getPref("clippings.entries.add_silently"));
    }
    catch (e) {}

    // Initialize "New From Clipboard" command on status bar icon menu.
    var ellipsis = this.showDialog ? this.strBundle.getString("ellipsis") : "";
    var newFromClpbdCmd = document.getElementById("ae_new_clipping_from_clpbd");
    newFromClpbdCmd.setAttribute("label",
				 this.strBundle.getString("newFromClipbd")
				 + ellipsis);

    // Initialize Clippings toolbar
    var toolbarEnabled = false;
    try {
      toolbarEnabled = this.appUtils.getPref("clippings.toolbar.enabled");
    }
    catch (e) {}

    if (toolbarEnabled) {
      this._clippingsToolbar = Components.classes["clippings@mozdev.org/clippings-toolbar;1"].getService(Components.interfaces.nsIClippingsToolbarService);
      this._clippingsToolbar.init(this.clippingsSvc);
      document.getElementById("ae-clippings-toggle-toolbar-2").removeAttribute("hidden");
    }
    else {
      this.focusHostAppWnd = function () {};
      this.initToolbarCmd = function () {};
    }

    this.isClippingsInitialized = true;
  },


  initContextMenuItem: function ()
  {
    var clippingsMenu1 = document.getElementById("ae-clippings-menu-1");
    if (this.appUtils.isCommonDataSourceEnabled()) {
      clippingsMenu1.builder.refresh();
    }
    clippingsMenu1.builder.rebuild();

    var popup = document.getElementById("ae-clippings-popup-1");
    var strBundle = document.getElementById("ae-clippings-strings");
    var ellipsis = this.showDialog ? this.strBundle.getString("ellipsis") : "";

    if (! this.dataSrcInitialized) {
      this.initClippingsPopup(popup, clippingsMenu1);
    }

    var addEntryCmd = document.getElementById("ae_new_clipping_from_selection");
    var selection = this.getSelectedText();
  
    addEntryCmd.setAttribute("disabled", selection == "");
    addEntryCmd.setAttribute("label", strBundle.getString("newFromSelect")
			     + ellipsis);
  },


  unload: function ()
  {
    this.clippingsSvc.removeListener(this._clippingsListener);
    this._clippingsListener = null;
  },


  focusHostAppWnd :  function ()
  {
    if (! gClippings.appUtils) return;

    var ww = Components.classes["@mozilla.org/embedcomp/window-watcher;1"]
                       .getService(Components.interfaces.nsIWindowWatcher);

    var wnd = ww.activeWindow;
    if (wnd instanceof ChromeWindow) {
      if (this._clippingsToolbar.isHidden()) {
	this._clippingsToolbar.open();
	this._clippingsToolbar.setHidden(false);
      }
    }
  }
};


//
// Event handler initialization
//

window.addEventListener("load",   gClippings, false);
window.addEventListener("unload", gClippings, false);
window.addEventListener("focus",  gClippings, false);
